"""
Provides a decorator that implements terraform's external program protocol for data sources.
https://www.terraform.io/docs/providers/external/data_source.html
"""

import json
import sys
from functools import wraps

def error(message):
    """
    Errors must create non-zero status codes and human-readable, ideally one-line, messages on stderr.
    """
    print(message, file=sys.stderr)
    sys.exit(1)


def validate(data):
    """
    Query data and result data must have keys who's values are strings.
    """
    if not isinstance(data, dict):
        error('Data must be a dictionary.')
    for value in data.values():
        if not isinstance(value, str):
            error('Values must be strings.')


def terraform_external_data(function):
    """
    Query data is received on stdin as a JSON object.
    Result data must be returned on stdout as a JSON object.

    The wrapped function must expect its first positional argument to be a dictionary of the query data.
    """
    @wraps(function)
    def wrapper(*args, **kwargs):
        query = json.loads(sys.stdin.read())
        validate(query)
        try:
            result = function(query, *args, **kwargs)
        except Exception as e:
            # Terraform wants one-line errors so we catch all exceptions and trim down to just the message (no trace).
            error(f'{type(e).__name__}: {e}')
        validate(result)
        sys.stdout.write(json.dumps(result))
    return wrapper



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="terraform-external-data",
    version="1.0.3",
    author="Juozas Kibildis",
    author_email="juozas.kibildis@zenitech.co.uk",
    description="Terraform external data package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/juozaskibildis-zt/terraform_external_data",
    project_urls={
        "Bug Tracker": "https://github.com/juozaskibildis-zt/terraform_external_data/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    python_requires=">=3.6",
)
